# --------------------------------------------------------------------------- #
#
#  hatena/bookmark/count.rb
#
#  Copyright (c) 2008, clown. All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    - No names of its contributors may be used to endorse or promote
#      products derived from this software without specific prior written
#      permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  Last-modified: Thu 04 Dec 2008 10:19:00 JST
#
# --------------------------------------------------------------------------- #

module Hatena
	module Bookmark
		# ------------------------------------------------------------------- #
		#  Hatena::Bookmark::Count
		# ------------------------------------------------------------------- #
		class Count
			require 'xmlrpc/client'
			require 'hatena/property'
			
			def initialize()
				@session = XMLRPC::Client.new(Hatena::Bookmark::END_POINT, '/xmlrpc').proxy('bookmark')
			end
			
			def entry(uri)
				result = nil
				
				case uri
				when String
					response = @session.getCount(Hatena::escape(uri))
					result = response[Hatena::escape(uri)].to_i
				when Array
					result = @session.getCount(*uri)
				end
				
				return result
			end
			
			def site(uri)
				return @session.getTotalCount(Hatena::escape(uri)).to_i
			end
			
			def asin(code)
				result = nil
				
				case code
				when String
					response = @session.getAsinCount(code)
					result = response[code].to_i
				when Array
					result = @session.getAsinCount(*code)
				end
				
				return result
			end
		end
		
		# ------------------------------------------------------------------- #
		#  Hatena::Bookmark::count
		# ------------------------------------------------------------------- #
		def count(uri)
			result = Count.new.entry(uri)
			if (block_given?)
				case result
				when String
					yield(uri, result)
				when Hash
					result.each { |s, count|
						yield(s, count.to_i)
					}
				end
			end
			return result
		end
		module_function :count
		
		# ------------------------------------------------------------------- #
		#  Hatena::Bookmark::site_count
		# ------------------------------------------------------------------- #
		def site_count(uri)
			result = Count.new.site(uri)
			yield(uri, result) if (block_given?)
			return result
		end
		module_function :site_count
		
		# ------------------------------------------------------------------- #
		#  Hatena::Bookmark::asin_count
		# ------------------------------------------------------------------- #
		def asin_count(code)
			result = Count.new.asin(code)
			if (block_given?)
				case result
				when String
					yield(code, result)
				when Hash
					result.each { |s, count|
						yield(s, count.to_i)
					}
				end
			end
			return result
		end
		module_function :asin_count
	end # Bookmark
end # Hatena
