# --------------------------------------------------------------------------- #
#
#  hatena/bookmark/distribution.rb
#
#  Copyright (c) 2008, clown. All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    - No names of its contributors may be used to endorse or promote
#      products derived from this software without specific prior written
#      permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  Last-modified: Sat 23 Aug 2008 14:52:00 JST
#
# --------------------------------------------------------------------------- #

module Hatena
	module Bookmark
		# ------------------------------------------------------------------- #
		#
		#  Hatena::Bookmark::Distribution
		#
		#  The Distribution class calcurates the number of usrs/tags
		#  encounted in given URIs, and puts the result in each user/tag.
		#
		# ------------------------------------------------------------------- #
		class Distribution
			require 'hatena/bookmark/entry'
			
			attr_reader :users
			attr_reader :tags
			attr_reader :sample
			
			def initialize()
				@session = Hatena::Bookmark::Entry.new
				@users = Hash.new
				@tags = Hash.new
				@sample = 0
			end
			
			def reset()
				@users.clear
				@tags.clear
				@sample = 0
				@session.reset
				return self
			end
			
			# get bookmark information and add the number of appearances
			# in each user/tag
			def add(uri)
				entry = @session.get(uri)
				return false if (entry == nil)
				
				entry.users.each { |key, value|
					if (@users.member?(key))
						@users[key] += 1
					else
						@users[key] = 1
					end
					
					value.tags.each { |tag|
						if (@tags.member?(tag))
							@tags[tag] += 1
						else
							@tags[tag] = 1
						end
					}
				}
				
				@sample += 1
				return true
			end
			
			def sort_users()
				return xdescending_sort(@users)
			end
			
			def sort_tags()
				return xdescending_sort(@tags)
			end
			
			# desceding sort by using the number of appearances
			def xdescending_sort(item)
				dest = item.to_a.sort { |x, y|
					y[1] <=> x[1]
				}
				return dest
			end
			private :xdescending_sort
		end
	end # Bookmark
end # Hatena
