# --------------------------------------------------------------------------- #
#
#  hatena/bookmark/entry.rb
#
#  Copyright (c) 2008, clown. All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    - No names of its contributors may be used to endorse or promote
#      products derived from this software without specific prior written
#      permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  Last-modified: Sun 16 Nov 2008 03:22:23 JST
#
# --------------------------------------------------------------------------- #

module Hatena
	module Bookmark
		# ------------------------------------------------------------------- #
		#  Hatena::Bookmark::Entry
		# ------------------------------------------------------------------- #
		class Entry
			require 'uri'
			require 'net/http'
			require 'rubygems'
			require 'json'
			
			require 'hatena/property'
			require 'hatena/utility'
			require 'hatena/session'
			
			Net::HTTP.version_1_2
			
			User     = Struct.new(:tags, :date, :comment)
			Related  = Struct.new(:eid, :uri, :title, :count)
			Response = Struct.new(:eid, :uri, :image_uri, :title, :count, :users, :related)
			
			def initialize(proxy = nil)
				host = nil
				port = nil
				if (proxy != nil)
					uri = URI.parse(proxy)
					host = uri.host
					port = uri.port
				end
				
				@session = Net::HTTP.new(Hatena::Bookmark::END_POINT, 80, host, port)
			end
			
			def reset()
				@session.finish if (@session.active?)
				@session.start
				return self
			end
			
			def get(uri)
				path = '/entry/json/?url=' + Hatena::escape(uri)
				
				begin
					result = Hatena::Session.get(@session, path, Hatena::MAX_RETRY)
					return nil if (result == nil || result.code.to_i != 200)
					json = JSON.parse(result.body)
					
					users = Hash.new
					json['bookmarks'].each { |item|
						tags = Array.new
						item['tags'].each { |tag|
							tags.push(tag)
						}
						date = Time.local(*item['timestamp'].split(/\W/))
						entry = User.new(tags, date, item['comment'])
						users[item['user']] = entry
					}
					
					related = Array.new()
					json['related'].each { |item|
						entry = Related.new(item['eid'].to_i, item['url'], item['title'], item['count'].to_i)
						related.push(entry)
					}
				rescue Exception
					return nil
				end
				
				return Response.new(json['eid'].to_i, json['url'], json['screenshot'],
					json['title'], json['count'].to_i, users, related)
			end
			
			def each(uri)
				case uri
				when String
					response = self.get(uri)
					yield(response)
				when Array
					uri.each { |s|
						response = self.get(s)
						yield(response)
					}
				end
				
				return self
			end
		end
	end # Bookmark
end # Hatena
