# --------------------------------------------------------------------------- #
#
#  hatena/bookmark/summary.rb
#
#  Copyright (c) 2008, clown. All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    - No names of its contributors may be used to endorse or promote
#      products derived from this software without specific prior written
#      permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  Last-modified: Thu 04 Dec 2008 12:30:00 JST
#
# --------------------------------------------------------------------------- #

module Hatena
	module Bookmark
		# ------------------------------------------------------------------- #
		#  Hatena::Bookmark::UserSummary
		# ------------------------------------------------------------------- #
		class UserSummary
			require 'net/http'
			require 'uri'
			
			require 'hatena/property'
			require 'hatena/session'
			
			Net::HTTP.version_1_2
			
			Response = Struct.new(:id, :uri, :title, :count, :favorite, :follower)
			
			def initialize(proxy = nil)
				host = nil
				port = nil
				if (proxy != nil)
					uri = URI.parse(proxy)
					host = uri.host
					port = uri.port
				end
				
				@session = Net::HTTP.new(Hatena::Bookmark::END_POINT, 80, host, port)
			end
			
			def reset()
				@session.finish if (@session.active?)
				@session.start
				return self
			end
			
			def get(id)
				path = "/#{id}/"
				uri = "http://#{Hatena::Bookmark::END_POINT}#{path}"
				
				result = Hatena::Session.get(@session, path, Hatena::MAX_RETRY)
				return nil if (result == nil || result.code.to_i != 200)
				
				title = ""
				count = 0
				favorite = 0
				follower = 0
				title = result.body().scan(/<h2><a.+?>.+?<\/a>(.+?)<\/h2>/m).to_s.strip!
				result.body().scan(/<div class=".*?hatena-module-profile">.*?<div class="hatena-module-body">(.+?)<\/div>/m) { |s|
					block = s.to_s
					count = block.scan(/<span class="label">ブックマーク数<\/span>(.+?)<\/li>/m).to_s.delete(" ,.")
					favorite = block.scan(/<span class="label">お気に入り<\/span>.*?<a .+?>(.+?)<\/a>/m).to_s.delete(" ,.")
					follower = block.scan(/<span class="label">お気に入られ<\/span>.*?<a .+?>(.+?)<\/a>/m).to_s.delete(" ,.")
					break
				}
				
				return Response.new(id, uri, title, count.to_i, favorite.to_i, follower.to_i)
			end
			
			def each(id)
				case id
				when String
					response = self.get(id)
					yield(response)
				when Array
					id.each { |s|
						response = self.get(s)
						yield(response)
					}
				end
				
				return self
			end
		end
	end # Bookmark
end # Hatena
