# --------------------------------------------------------------------------- #
#
#  hatena/bookmark/trace.rb
#
#  Copyright (c) 2008, clown. All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#    - Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    - No names of its contributors may be used to endorse or promote
#      products derived from this software without specific prior written
#      permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
#  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
#  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
#  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
#  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  Last-modified: Thu 18 Dec 2008 07:26:00 JST
#
# --------------------------------------------------------------------------- #

module Hatena
	module Bookmark
		# ------------------------------------------------------------------- #
		#
		#  Hatena::Bookmark::URITrace
		#
		#  The URITrace class gets URIs of the hatena bookmark page. The
		#  class continues to getting new URIs based on the "next" meta
		#  tag of target html pages.
		#
		# ------------------------------------------------------------------- #
		class URITrace
			require 'net/http'
			require 'uri'
			
			require 'hatena/property'
			require 'hatena/session'
			
			Net::HTTP.version_1_2
			
			def initialize(path, proxy = nil)
				@base = String.new
				@base.concat("/") if (path[0] != '/'.to_i)
				@base.concat(path)
				
				if (@base.match(/^\/[\w\-]+$/) != nil &&
					@base != "/hotentry" &&
					@base != "/entrylist" &&
					@base != "/news" &&
					@base != "/video" &&
					@base != "/asin")
					@base.concat("/")
				end
				
				@path = String.new(@base)
				
				host = nil
				port = nil
				if (proxy != nil)
					uri = URI.parse(proxy)
					host = uri.host
					port = uri.port
				end
				@session = Net::HTTP.new(Hatena::Bookmark::END_POINT, 80, host, port)
			end
			
			def reset()
				@path = String.new(@base)
				@session.finish if (@session.active?)
				@session.start
				return self
			end
			
			def get(limit = nil)
				result = Array.new()
				
				sum = 0
				while (@path != nil && (limit == nil || sum < limit))
					response = Hatena::Session::get(@session, @path, Hatena::MAX_RETRY)
					if (response == nil || response.code.to_i != 200)
						return result if (result.length > 0)
						return nil
					end
					
					count = 0
					link = response.body().scan(/<link rel="next" href="(.+?)".*?>/m).to_s
					response.body().scan(/<h3.*?>(.+?)<\/h3>/m) { |block|
						block.to_s.scan(/<a href="(.+?)".*?>.+?<\/a>/m) { |uri|
							result.push(uri.to_s)
							count += 1
							break
						}
						break if (limit != nil && sum + count >= limit)
					}
					sum += count
					
					if (link.empty?)
						@path = nil
					elsif (link[0] == 47)
						@path = link
					elsif (link[0] == 63)
						@path.gsub!(/\?.*$/, "")
						@path.concat(link)
					else
						@path = @base + link
					end
					
					break if (limit == nil)
				end
				
				return nil if (result.empty?)
				return result
			end
			
			def each(limit = nil)
				n = 0
				while (limit == nil || n < limit) do
					result = self.get
					break if (result == nil || result.length == 0)
					result.each { |uri|
						yield(uri)
						n += 1
						break if (limit != nil && n >= limit)
					}
				end
				
				return self
			end
		end
	end # Bookmark
end # Hatena
